# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

TCKSAMPLE_METADATA = Metadata(
    id="274c00456de615fd846529a52bdcb4684c1c52eb.boutiques",
    name="tcksample",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_TcksampleConfigParamsDictNoTag = typing.TypedDict('_TcksampleConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
TcksampleConfigParamsDictTagged = typing.TypedDict('TcksampleConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
TcksampleConfigParamsDict = _TcksampleConfigParamsDictNoTag | TcksampleConfigParamsDictTagged


_TcksampleParamsDictNoTag = typing.TypedDict('_TcksampleParamsDictNoTag', {
    "stat_tck": typing.NotRequired[str | None],
    "nointerp": bool,
    "precise": bool,
    "use_tdi_fraction": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TcksampleConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "tracks": InputPathType,
    "image": InputPathType,
    "values": str,
})
TcksampleParamsDictTagged = typing.TypedDict('TcksampleParamsDictTagged', {
    "@type": typing.Literal["mrtrix/tcksample"],
    "stat_tck": typing.NotRequired[str | None],
    "nointerp": bool,
    "precise": bool,
    "use_tdi_fraction": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TcksampleConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "tracks": InputPathType,
    "image": InputPathType,
    "values": str,
})
TcksampleParamsDict = _TcksampleParamsDictNoTag | TcksampleParamsDictTagged


def tcksample_config(
    key: str,
    value: str,
) -> TcksampleConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def tcksample_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TcksampleConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def tcksample_config_cargs(
    params: TcksampleConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class TcksampleOutputs(typing.NamedTuple):
    """
    Output object returned when calling `TcksampleParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    values_: OutputPathType
    """the output sampled values"""


def tcksample_params(
    tracks: InputPathType,
    image: InputPathType,
    values_: str,
    stat_tck: str | None = None,
    nointerp: bool = False,
    precise: bool = False,
    use_tdi_fraction: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TcksampleConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> TcksampleParamsDictTagged:
    """
    Build parameters.
    
    Args:
        tracks: the input track file.
        image: the image to be sampled.
        values_: the output sampled values.
        stat_tck: compute some statistic from the values along each streamline\
            (options are: mean,median,min,max).
        nointerp: do not use trilinear interpolation when sampling image values.
        precise: use the precise mechanism for mapping streamlines to voxels\
            (obviates the need for trilinear interpolation) (only applicable if\
            some per-streamline statistic is requested).
        use_tdi_fraction: each streamline is assigned a fraction of the image\
            intensity in each voxel based on the fraction of the track density\
            contributed by that streamline (this is only appropriate for processing\
            a whole-brain tractogram, and images for which the quantiative\
            parameter is additive).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/tcksample",
        "nointerp": nointerp,
        "precise": precise,
        "use_tdi_fraction": use_tdi_fraction,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "tracks": tracks,
        "image": image,
        "values": values_,
    }
    if stat_tck is not None:
        params["stat_tck"] = stat_tck
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def tcksample_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TcksampleParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("stat_tck", None) is not None:
        if not isinstance(params["stat_tck"], str):
            raise StyxValidationError(f'`stat_tck` has the wrong type: Received `{type(params.get("stat_tck", None))}` expected `str | None`')
    if params.get("nointerp", False) is None:
        raise StyxValidationError("`nointerp` must not be None")
    if not isinstance(params["nointerp"], bool):
        raise StyxValidationError(f'`nointerp` has the wrong type: Received `{type(params.get("nointerp", False))}` expected `bool`')
    if params.get("precise", False) is None:
        raise StyxValidationError("`precise` must not be None")
    if not isinstance(params["precise"], bool):
        raise StyxValidationError(f'`precise` has the wrong type: Received `{type(params.get("precise", False))}` expected `bool`')
    if params.get("use_tdi_fraction", False) is None:
        raise StyxValidationError("`use_tdi_fraction` must not be None")
    if not isinstance(params["use_tdi_fraction"], bool):
        raise StyxValidationError(f'`use_tdi_fraction` has the wrong type: Received `{type(params.get("use_tdi_fraction", False))}` expected `bool`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[TcksampleConfigParamsDict] | None`')
        for e in params["config"]:
            tcksample_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("tracks", None) is None:
        raise StyxValidationError("`tracks` must not be None")
    if not isinstance(params["tracks"], (pathlib.Path, str)):
        raise StyxValidationError(f'`tracks` has the wrong type: Received `{type(params.get("tracks", None))}` expected `InputPathType`')
    if params.get("image", None) is None:
        raise StyxValidationError("`image` must not be None")
    if not isinstance(params["image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`image` has the wrong type: Received `{type(params.get("image", None))}` expected `InputPathType`')
    if params.get("values", None) is None:
        raise StyxValidationError("`values` must not be None")
    if not isinstance(params["values"], str):
        raise StyxValidationError(f'`values` has the wrong type: Received `{type(params.get("values", None))}` expected `str`')


def tcksample_cargs(
    params: TcksampleParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("tcksample")
    if params.get("stat_tck", None) is not None:
        cargs.extend([
            "-stat_tck",
            params.get("stat_tck", None)
        ])
    if params.get("nointerp", False):
        cargs.append("-nointerp")
    if params.get("precise", False):
        cargs.append("-precise")
    if params.get("use_tdi_fraction", False):
        cargs.append("-use_tdi_fraction")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [tcksample_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("tracks", None)))
    cargs.append(execution.input_file(params.get("image", None)))
    cargs.append(params.get("values", None))
    return cargs


def tcksample_outputs(
    params: TcksampleParamsDict,
    execution: Execution,
) -> TcksampleOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = TcksampleOutputs(
        root=execution.output_file("."),
        values_=execution.output_file(params.get("values", None)),
    )
    return ret


def tcksample_execute(
    params: TcksampleParamsDict,
    runner: Runner | None = None,
) -> TcksampleOutputs:
    """
    tcksample
    
    Sample values of an associated image along tracks.
    
    By default, the value of the underlying image at each point along the track
    is written to either an ASCII file (with all values for each track on the
    same line), or a track scalar file (.tsf). Alternatively, some statistic can
    be taken from the values along each streamline and written to a vector file.
    
    References:
    
    * If using -precise option: Smith, R. E.; Tournier, J.-D.; Calamante, F. &
    Connelly, A. SIFT: Spherical-deconvolution informed filtering of
    tractograms. NeuroImage, 2013, 67, 298-312.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TcksampleOutputs`).
    """
    tcksample_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(TCKSAMPLE_METADATA)
    params = execution.params(params)
    cargs = tcksample_cargs(params, execution)
    ret = tcksample_outputs(params, execution)
    execution.run(cargs)
    return ret


def tcksample(
    tracks: InputPathType,
    image: InputPathType,
    values_: str,
    stat_tck: str | None = None,
    nointerp: bool = False,
    precise: bool = False,
    use_tdi_fraction: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TcksampleConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> TcksampleOutputs:
    """
    tcksample
    
    Sample values of an associated image along tracks.
    
    By default, the value of the underlying image at each point along the track
    is written to either an ASCII file (with all values for each track on the
    same line), or a track scalar file (.tsf). Alternatively, some statistic can
    be taken from the values along each streamline and written to a vector file.
    
    References:
    
    * If using -precise option: Smith, R. E.; Tournier, J.-D.; Calamante, F. &
    Connelly, A. SIFT: Spherical-deconvolution informed filtering of
    tractograms. NeuroImage, 2013, 67, 298-312.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        tracks: the input track file.
        image: the image to be sampled.
        values_: the output sampled values.
        stat_tck: compute some statistic from the values along each streamline\
            (options are: mean,median,min,max).
        nointerp: do not use trilinear interpolation when sampling image values.
        precise: use the precise mechanism for mapping streamlines to voxels\
            (obviates the need for trilinear interpolation) (only applicable if\
            some per-streamline statistic is requested).
        use_tdi_fraction: each streamline is assigned a fraction of the image\
            intensity in each voxel based on the fraction of the track density\
            contributed by that streamline (this is only appropriate for processing\
            a whole-brain tractogram, and images for which the quantiative\
            parameter is additive).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TcksampleOutputs`).
    """
    params = tcksample_params(
        stat_tck=stat_tck,
        nointerp=nointerp,
        precise=precise,
        use_tdi_fraction=use_tdi_fraction,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        tracks=tracks,
        image=image,
        values_=values_,
    )
    return tcksample_execute(params, runner)


__all__ = [
    "TCKSAMPLE_METADATA",
    "TcksampleConfigParamsDict",
    "TcksampleConfigParamsDictTagged",
    "TcksampleOutputs",
    "TcksampleParamsDict",
    "TcksampleParamsDictTagged",
    "tcksample",
    "tcksample_config",
    "tcksample_execute",
    "tcksample_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

MRGRID_METADATA = Metadata(
    id="62aab6d384f12d5080514505c5f5b4c9d7d75bc0.boutiques",
    name="mrgrid",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_MrgridAxisParamsDictNoTag = typing.TypedDict('_MrgridAxisParamsDictNoTag', {
    "index": int,
    "spec": str,
})
MrgridAxisParamsDictTagged = typing.TypedDict('MrgridAxisParamsDictTagged', {
    "@type": typing.Literal["axis"],
    "index": int,
    "spec": str,
})
MrgridAxisParamsDict = _MrgridAxisParamsDictNoTag | MrgridAxisParamsDictTagged


_MrgridVariousStringParamsDictNoTag = typing.TypedDict('_MrgridVariousStringParamsDictNoTag', {
    "obj": str,
})
MrgridVariousStringParamsDictTagged = typing.TypedDict('MrgridVariousStringParamsDictTagged', {
    "@type": typing.Literal["VariousString"],
    "obj": str,
})
MrgridVariousStringParamsDict = _MrgridVariousStringParamsDictNoTag | MrgridVariousStringParamsDictTagged


_MrgridVariousFileParamsDictNoTag = typing.TypedDict('_MrgridVariousFileParamsDictNoTag', {
    "obj": InputPathType,
})
MrgridVariousFileParamsDictTagged = typing.TypedDict('MrgridVariousFileParamsDictTagged', {
    "@type": typing.Literal["VariousFile"],
    "obj": InputPathType,
})
MrgridVariousFileParamsDict = _MrgridVariousFileParamsDictNoTag | MrgridVariousFileParamsDictTagged


_MrgridConfigParamsDictNoTag = typing.TypedDict('_MrgridConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
MrgridConfigParamsDictTagged = typing.TypedDict('MrgridConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
MrgridConfigParamsDict = _MrgridConfigParamsDictNoTag | MrgridConfigParamsDictTagged


_MrgridParamsDictNoTag = typing.TypedDict('_MrgridParamsDictNoTag', {
    "template": typing.NotRequired[InputPathType | None],
    "size": typing.NotRequired[list[int] | None],
    "voxel": typing.NotRequired[list[float] | None],
    "scale": typing.NotRequired[list[float] | None],
    "interp": typing.NotRequired[str | None],
    "oversample": typing.NotRequired[list[int] | None],
    "as": typing.NotRequired[InputPathType | None],
    "uniform": typing.NotRequired[int | None],
    "mask": typing.NotRequired[InputPathType | None],
    "crop_unbound": bool,
    "axis": typing.NotRequired[list[MrgridAxisParamsDict] | None],
    "all_axes": bool,
    "fill": typing.NotRequired[float | None],
    "strides": typing.NotRequired[typing.Union[MrgridVariousStringParamsDictTagged, MrgridVariousFileParamsDictTagged] | None],
    "datatype": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrgridConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "operation": str,
    "output": str,
})
MrgridParamsDictTagged = typing.TypedDict('MrgridParamsDictTagged', {
    "@type": typing.Literal["mrtrix/mrgrid"],
    "template": typing.NotRequired[InputPathType | None],
    "size": typing.NotRequired[list[int] | None],
    "voxel": typing.NotRequired[list[float] | None],
    "scale": typing.NotRequired[list[float] | None],
    "interp": typing.NotRequired[str | None],
    "oversample": typing.NotRequired[list[int] | None],
    "as": typing.NotRequired[InputPathType | None],
    "uniform": typing.NotRequired[int | None],
    "mask": typing.NotRequired[InputPathType | None],
    "crop_unbound": bool,
    "axis": typing.NotRequired[list[MrgridAxisParamsDict] | None],
    "all_axes": bool,
    "fill": typing.NotRequired[float | None],
    "strides": typing.NotRequired[typing.Union[MrgridVariousStringParamsDictTagged, MrgridVariousFileParamsDictTagged] | None],
    "datatype": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrgridConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "operation": str,
    "output": str,
})
MrgridParamsDict = _MrgridParamsDictNoTag | MrgridParamsDictTagged


def mrgrid_strides_cargs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "VariousString": mrgrid_various_string_cargs,
        "VariousFile": mrgrid_various_file_cargs,
    }.get(t)


def mrgrid_strides_validate_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get validate params function by command type.
    
    Args:
        t: Command type.
    Returns:
        Validate params function.
    """
    return {
        "VariousString": mrgrid_various_string_validate,
        "VariousFile": mrgrid_various_file_validate,
    }.get(t)


def mrgrid_axis(
    index: int,
    spec: str,
) -> MrgridAxisParamsDictTagged:
    """
    Build parameters.
    
    Args:
        index: pad or crop the input image along the provided axis (defined by\
            index). The specifier argument defines the number of voxels added or\
            removed on the lower or upper end of the axis (-axis index\
            delta_lower,delta_upper) or acts as a voxel selection range (-axis\
            index start:stop). In both modes, values are relative to the input\
            image (overriding all other extent-specifying options). Negative delta\
            specifier values trigger the inverse operation (pad instead of crop and\
            vice versa) and negative range specifier trigger padding. Note that the\
            deprecated commands 'mrcrop' and 'mrpad' used range-based and\
            delta-based -axis indices, respectively.
        spec: pad or crop the input image along the provided axis (defined by\
            index). The specifier argument defines the number of voxels added or\
            removed on the lower or upper end of the axis (-axis index\
            delta_lower,delta_upper) or acts as a voxel selection range (-axis\
            index start:stop). In both modes, values are relative to the input\
            image (overriding all other extent-specifying options). Negative delta\
            specifier values trigger the inverse operation (pad instead of crop and\
            vice versa) and negative range specifier trigger padding. Note that the\
            deprecated commands 'mrcrop' and 'mrpad' used range-based and\
            delta-based -axis indices, respectively.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "axis",
        "index": index,
        "spec": spec,
    }
    return params


def mrgrid_axis_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrgridAxisParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("index", None) is None:
        raise StyxValidationError("`index` must not be None")
    if not isinstance(params["index"], int):
        raise StyxValidationError(f'`index` has the wrong type: Received `{type(params.get("index", None))}` expected `int`')
    if params.get("spec", None) is None:
        raise StyxValidationError("`spec` must not be None")
    if not isinstance(params["spec"], str):
        raise StyxValidationError(f'`spec` has the wrong type: Received `{type(params.get("spec", None))}` expected `str`')


def mrgrid_axis_cargs(
    params: MrgridAxisParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-axis")
    cargs.append(str(params.get("index", None)))
    cargs.append(params.get("spec", None))
    return cargs


def mrgrid_various_string(
    obj: str,
) -> MrgridVariousStringParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: String object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousString",
        "obj": obj,
    }
    return params


def mrgrid_various_string_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrgridVariousStringParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], str):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `str`')


def mrgrid_various_string_cargs(
    params: MrgridVariousStringParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(params.get("obj", None))
    return cargs


def mrgrid_various_file(
    obj: InputPathType,
) -> MrgridVariousFileParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: File object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousFile",
        "obj": obj,
    }
    return params


def mrgrid_various_file_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrgridVariousFileParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], (pathlib.Path, str)):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `InputPathType`')


def mrgrid_various_file_cargs(
    params: MrgridVariousFileParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("obj", None)))
    return cargs


def mrgrid_config(
    key: str,
    value: str,
) -> MrgridConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def mrgrid_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrgridConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def mrgrid_config_cargs(
    params: MrgridConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class MrgridOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrgridParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output image."""


def mrgrid_params(
    input_: InputPathType,
    operation: str,
    output: str,
    template: InputPathType | None = None,
    size: list[int] | None = None,
    voxel: list[float] | None = None,
    scale: list[float] | None = None,
    interp: str | None = None,
    oversample: list[int] | None = None,
    as_: InputPathType | None = None,
    uniform: int | None = None,
    mask: InputPathType | None = None,
    crop_unbound: bool = False,
    axis: list[MrgridAxisParamsDict] | None = None,
    all_axes: bool = False,
    fill: float | None = None,
    strides: typing.Union[MrgridVariousStringParamsDictTagged, MrgridVariousFileParamsDictTagged] | None = None,
    datatype: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrgridConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> MrgridParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: input image to be regridded.
        operation: the operation to be performed, one of: regrid, crop, pad.
        output: the output image.
        template: match the input image grid (voxel spacing, image size, header\
            transformation) to that of a reference image. The image resolution\
            relative to the template image can be changed with one of -size,\
            -voxel, -scale.
        size: define the size (number of voxels) in each spatial dimension for\
            the output image. This should be specified as a comma-separated list.
        voxel: define the new voxel size for the output image. This can be\
            specified either as a single value to be used for all spatial\
            dimensions, or as a comma-separated list of the size for each voxel\
            dimension.
        scale: scale the image resolution by the supplied factor. This can be\
            specified either as a single value to be used for all dimensions, or as\
            a comma-separated list of scale factors for each dimension.
        interp: set the interpolation method to use when reslicing (choices:\
            nearest, linear, cubic, sinc. Default: cubic).
        oversample: set the amount of over-sampling (in the target space) to\
            perform when regridding. This is particularly relevant when downsamping\
            a high-resolution image to a low-resolution image, to avoid aliasing\
            artefacts. This can consist of a single integer, or a comma-separated\
            list of 3 integers if different oversampling factors are desired along\
            the different axes. Default is determined from ratio of voxel\
            dimensions (disabled for nearest-neighbour interpolation).
        as_: pad or crop the input image on the upper bound to match the\
            specified reference image grid. This operation ignores differences in\
            image transformation between input and reference image.
        uniform: pad or crop the input image by a uniform number of voxels on\
            all sides.
        mask: crop the input image according to the spatial extent of a mask\
            image. The mask must share a common voxel grid with the input image but\
            differences in image transformations are ignored. Note that even though\
            only 3 dimensions are cropped when using a mask, the bounds are\
            computed by checking the extent for all dimensions. Note that by\
            default a gap of 1 voxel is left at all edges of the image to allow\
            valid trilinear interpolation. This gap can be modified with the\
            -uniform option but by default it does not extend beyond the FOV unless\
            -crop_unbound is used.
        crop_unbound: Allow padding beyond the original FOV when cropping.
        axis: pad or crop the input image along the provided axis (defined by\
            index). The specifier argument defines the number of voxels added or\
            removed on the lower or upper end of the axis (-axis index\
            delta_lower,delta_upper) or acts as a voxel selection range (-axis\
            index start:stop). In both modes, values are relative to the input\
            image (overriding all other extent-specifying options). Negative delta\
            specifier values trigger the inverse operation (pad instead of crop and\
            vice versa) and negative range specifier trigger padding. Note that the\
            deprecated commands 'mrcrop' and 'mrpad' used range-based and\
            delta-based -axis indices, respectively.
        all_axes: Crop or pad all, not just spatial axes.
        fill: Use number as the out of bounds value. nan, inf and -inf are\
            valid arguments. (Default: 0.0).
        strides: specify the strides of the output data in memory; either as a\
            comma-separated list of (signed) integers, or as a template image from\
            which the strides shall be extracted and used. The actual strides\
            produced will depend on whether the output image format can support it.
        datatype: specify output image data type. Valid choices are: float32,\
            float32le, float32be, float64, float64le, float64be, int64, uint64,\
            int64le, uint64le, int64be, uint64be, int32, uint32, int32le, uint32le,\
            int32be, uint32be, int16, uint16, int16le, uint16le, int16be, uint16be,\
            cfloat32, cfloat32le, cfloat32be, cfloat64, cfloat64le, cfloat64be,\
            int8, uint8, bit.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/mrgrid",
        "crop_unbound": crop_unbound,
        "all_axes": all_axes,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
        "operation": operation,
        "output": output,
    }
    if template is not None:
        params["template"] = template
    if size is not None:
        params["size"] = size
    if voxel is not None:
        params["voxel"] = voxel
    if scale is not None:
        params["scale"] = scale
    if interp is not None:
        params["interp"] = interp
    if oversample is not None:
        params["oversample"] = oversample
    if as_ is not None:
        params["as"] = as_
    if uniform is not None:
        params["uniform"] = uniform
    if mask is not None:
        params["mask"] = mask
    if axis is not None:
        params["axis"] = axis
    if fill is not None:
        params["fill"] = fill
    if strides is not None:
        params["strides"] = strides
    if datatype is not None:
        params["datatype"] = datatype
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def mrgrid_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrgridParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("template", None) is not None:
        if not isinstance(params["template"], (pathlib.Path, str)):
            raise StyxValidationError(f'`template` has the wrong type: Received `{type(params.get("template", None))}` expected `InputPathType | None`')
    if params.get("size", None) is not None:
        if not isinstance(params["size"], list):
            raise StyxValidationError(f'`size` has the wrong type: Received `{type(params.get("size", None))}` expected `list[int] | None`')
        for e in params["size"]:
            if not isinstance(e, int):
                raise StyxValidationError(f'`size` has the wrong type: Received `{type(params.get("size", None))}` expected `list[int] | None`')
    if params.get("voxel", None) is not None:
        if not isinstance(params["voxel"], list):
            raise StyxValidationError(f'`voxel` has the wrong type: Received `{type(params.get("voxel", None))}` expected `list[float] | None`')
        for e in params["voxel"]:
            if not isinstance(e, (float, int)):
                raise StyxValidationError(f'`voxel` has the wrong type: Received `{type(params.get("voxel", None))}` expected `list[float] | None`')
    if params.get("scale", None) is not None:
        if not isinstance(params["scale"], list):
            raise StyxValidationError(f'`scale` has the wrong type: Received `{type(params.get("scale", None))}` expected `list[float] | None`')
        for e in params["scale"]:
            if not isinstance(e, (float, int)):
                raise StyxValidationError(f'`scale` has the wrong type: Received `{type(params.get("scale", None))}` expected `list[float] | None`')
    if params.get("interp", None) is not None:
        if not isinstance(params["interp"], str):
            raise StyxValidationError(f'`interp` has the wrong type: Received `{type(params.get("interp", None))}` expected `str | None`')
    if params.get("oversample", None) is not None:
        if not isinstance(params["oversample"], list):
            raise StyxValidationError(f'`oversample` has the wrong type: Received `{type(params.get("oversample", None))}` expected `list[int] | None`')
        for e in params["oversample"]:
            if not isinstance(e, int):
                raise StyxValidationError(f'`oversample` has the wrong type: Received `{type(params.get("oversample", None))}` expected `list[int] | None`')
    if params.get("as", None) is not None:
        if not isinstance(params["as"], (pathlib.Path, str)):
            raise StyxValidationError(f'`as` has the wrong type: Received `{type(params.get("as", None))}` expected `InputPathType | None`')
    if params.get("uniform", None) is not None:
        if not isinstance(params["uniform"], int):
            raise StyxValidationError(f'`uniform` has the wrong type: Received `{type(params.get("uniform", None))}` expected `int | None`')
    if params.get("mask", None) is not None:
        if not isinstance(params["mask"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType | None`')
    if params.get("crop_unbound", False) is None:
        raise StyxValidationError("`crop_unbound` must not be None")
    if not isinstance(params["crop_unbound"], bool):
        raise StyxValidationError(f'`crop_unbound` has the wrong type: Received `{type(params.get("crop_unbound", False))}` expected `bool`')
    if params.get("axis", None) is not None:
        if not isinstance(params["axis"], list):
            raise StyxValidationError(f'`axis` has the wrong type: Received `{type(params.get("axis", None))}` expected `list[MrgridAxisParamsDict] | None`')
        for e in params["axis"]:
            mrgrid_axis_validate(e)
    if params.get("all_axes", False) is None:
        raise StyxValidationError("`all_axes` must not be None")
    if not isinstance(params["all_axes"], bool):
        raise StyxValidationError(f'`all_axes` has the wrong type: Received `{type(params.get("all_axes", False))}` expected `bool`')
    if params.get("fill", None) is not None:
        if not isinstance(params["fill"], (float, int)):
            raise StyxValidationError(f'`fill` has the wrong type: Received `{type(params.get("fill", None))}` expected `float | None`')
    if params.get("strides", None) is not None:
        if not isinstance(params["strides"], dict):
            raise StyxValidationError(f'Params object has the wrong type \'{type(params["strides"])}\'')
        if "@type" not in params["strides"]:
            raise StyxValidationError("Params object is missing `@type`")
        if params["strides"]["@type"] not in ["VariousString", "VariousFile"]:
            raise StyxValidationError("Parameter `strides`s `@type` must be one of [\"VariousString\", \"VariousFile\"]")
        mrgrid_strides_validate_dyn_fn(params["strides"]["@type"])(params["strides"])
    if params.get("datatype", None) is not None:
        if not isinstance(params["datatype"], str):
            raise StyxValidationError(f'`datatype` has the wrong type: Received `{type(params.get("datatype", None))}` expected `str | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[MrgridConfigParamsDict] | None`')
        for e in params["config"]:
            mrgrid_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("operation", None) is None:
        raise StyxValidationError("`operation` must not be None")
    if not isinstance(params["operation"], str):
        raise StyxValidationError(f'`operation` has the wrong type: Received `{type(params.get("operation", None))}` expected `str`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def mrgrid_cargs(
    params: MrgridParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("mrgrid")
    if params.get("template", None) is not None:
        cargs.extend([
            "-template",
            execution.input_file(params.get("template", None))
        ])
    if params.get("size", None) is not None:
        cargs.extend([
            "-size",
            ",".join(map(str, params.get("size", None)))
        ])
    if params.get("voxel", None) is not None:
        cargs.extend([
            "-voxel",
            ",".join(map(str, params.get("voxel", None)))
        ])
    if params.get("scale", None) is not None:
        cargs.extend([
            "-scale",
            ",".join(map(str, params.get("scale", None)))
        ])
    if params.get("interp", None) is not None:
        cargs.extend([
            "-interp",
            params.get("interp", None)
        ])
    if params.get("oversample", None) is not None:
        cargs.extend([
            "-oversample",
            ",".join(map(str, params.get("oversample", None)))
        ])
    if params.get("as", None) is not None:
        cargs.extend([
            "-as",
            execution.input_file(params.get("as", None))
        ])
    if params.get("uniform", None) is not None:
        cargs.extend([
            "-uniform",
            str(params.get("uniform", None))
        ])
    if params.get("mask", None) is not None:
        cargs.extend([
            "-mask",
            execution.input_file(params.get("mask", None))
        ])
    if params.get("crop_unbound", False):
        cargs.append("-crop_unbound")
    if params.get("axis", None) is not None:
        cargs.extend([a for c in [mrgrid_axis_cargs(s, execution) for s in params.get("axis", None)] for a in c])
    if params.get("all_axes", False):
        cargs.append("-all_axes")
    if params.get("fill", None) is not None:
        cargs.extend([
            "-fill",
            str(params.get("fill", None))
        ])
    if params.get("strides", None) is not None:
        cargs.extend([
            "-strides",
            *mrgrid_strides_cargs_dyn_fn(params.get("strides", None)["@type"])(params.get("strides", None), execution)
        ])
    if params.get("datatype", None) is not None:
        cargs.extend([
            "-datatype",
            params.get("datatype", None)
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [mrgrid_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(params.get("operation", None))
    cargs.append(params.get("output", None))
    return cargs


def mrgrid_outputs(
    params: MrgridParamsDict,
    execution: Execution,
) -> MrgridOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MrgridOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def mrgrid_execute(
    params: MrgridParamsDict,
    runner: Runner | None = None,
) -> MrgridOutputs:
    """
    mrgrid
    
    Modify the grid of an image without interpolation (cropping or padding) or
    by regridding to an image grid with modified orientation, location and or
    resolution. The image content remains in place in real world coordinates..
    
    - regrid: This operation performs changes of the voxel grid that require
    interpolation of the image such as changing the resolution or location and
    orientation of the voxel grid. If the image is down-sampled, the appropriate
    smoothing is automatically applied using Gaussian smoothing unless nearest
    neighbour interpolation is selected or oversample is changed explicitly. The
    resolution can only be changed for spatial dimensions.
    
    - crop: The image extent after cropping, can be specified either manually
    for each axis dimensions, or via a mask or reference image. The image can be
    cropped to the extent of a mask. This is useful for axially-acquired brain
    images, where the image size can be reduced by a factor of 2 by removing the
    empty space on either side of the brain. Note that cropping does not extend
    the image beyond the original FOV unless explicitly specified (via
    -crop_unbound or negative -axis extent).
    
    - pad: Analogously to cropping, padding increases the FOV of an image
    without image interpolation. Pad and crop can be performed simultaneously by
    specifying signed specifier argument values to the -axis option.
    
    This command encapsulates and extends the functionality of the superseded
    commands 'mrpad', 'mrcrop' and 'mrresize'. Note the difference in -axis
    convention used for 'mrcrop' and 'mrpad' (see -axis option description).
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrgridOutputs`).
    """
    mrgrid_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRGRID_METADATA)
    params = execution.params(params)
    cargs = mrgrid_cargs(params, execution)
    ret = mrgrid_outputs(params, execution)
    execution.run(cargs)
    return ret


def mrgrid(
    input_: InputPathType,
    operation: str,
    output: str,
    template: InputPathType | None = None,
    size: list[int] | None = None,
    voxel: list[float] | None = None,
    scale: list[float] | None = None,
    interp: str | None = None,
    oversample: list[int] | None = None,
    as_: InputPathType | None = None,
    uniform: int | None = None,
    mask: InputPathType | None = None,
    crop_unbound: bool = False,
    axis: list[MrgridAxisParamsDict] | None = None,
    all_axes: bool = False,
    fill: float | None = None,
    strides: typing.Union[MrgridVariousStringParamsDictTagged, MrgridVariousFileParamsDictTagged] | None = None,
    datatype: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrgridConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrgridOutputs:
    """
    mrgrid
    
    Modify the grid of an image without interpolation (cropping or padding) or
    by regridding to an image grid with modified orientation, location and or
    resolution. The image content remains in place in real world coordinates..
    
    - regrid: This operation performs changes of the voxel grid that require
    interpolation of the image such as changing the resolution or location and
    orientation of the voxel grid. If the image is down-sampled, the appropriate
    smoothing is automatically applied using Gaussian smoothing unless nearest
    neighbour interpolation is selected or oversample is changed explicitly. The
    resolution can only be changed for spatial dimensions.
    
    - crop: The image extent after cropping, can be specified either manually
    for each axis dimensions, or via a mask or reference image. The image can be
    cropped to the extent of a mask. This is useful for axially-acquired brain
    images, where the image size can be reduced by a factor of 2 by removing the
    empty space on either side of the brain. Note that cropping does not extend
    the image beyond the original FOV unless explicitly specified (via
    -crop_unbound or negative -axis extent).
    
    - pad: Analogously to cropping, padding increases the FOV of an image
    without image interpolation. Pad and crop can be performed simultaneously by
    specifying signed specifier argument values to the -axis option.
    
    This command encapsulates and extends the functionality of the superseded
    commands 'mrpad', 'mrcrop' and 'mrresize'. Note the difference in -axis
    convention used for 'mrcrop' and 'mrpad' (see -axis option description).
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: input image to be regridded.
        operation: the operation to be performed, one of: regrid, crop, pad.
        output: the output image.
        template: match the input image grid (voxel spacing, image size, header\
            transformation) to that of a reference image. The image resolution\
            relative to the template image can be changed with one of -size,\
            -voxel, -scale.
        size: define the size (number of voxels) in each spatial dimension for\
            the output image. This should be specified as a comma-separated list.
        voxel: define the new voxel size for the output image. This can be\
            specified either as a single value to be used for all spatial\
            dimensions, or as a comma-separated list of the size for each voxel\
            dimension.
        scale: scale the image resolution by the supplied factor. This can be\
            specified either as a single value to be used for all dimensions, or as\
            a comma-separated list of scale factors for each dimension.
        interp: set the interpolation method to use when reslicing (choices:\
            nearest, linear, cubic, sinc. Default: cubic).
        oversample: set the amount of over-sampling (in the target space) to\
            perform when regridding. This is particularly relevant when downsamping\
            a high-resolution image to a low-resolution image, to avoid aliasing\
            artefacts. This can consist of a single integer, or a comma-separated\
            list of 3 integers if different oversampling factors are desired along\
            the different axes. Default is determined from ratio of voxel\
            dimensions (disabled for nearest-neighbour interpolation).
        as_: pad or crop the input image on the upper bound to match the\
            specified reference image grid. This operation ignores differences in\
            image transformation between input and reference image.
        uniform: pad or crop the input image by a uniform number of voxels on\
            all sides.
        mask: crop the input image according to the spatial extent of a mask\
            image. The mask must share a common voxel grid with the input image but\
            differences in image transformations are ignored. Note that even though\
            only 3 dimensions are cropped when using a mask, the bounds are\
            computed by checking the extent for all dimensions. Note that by\
            default a gap of 1 voxel is left at all edges of the image to allow\
            valid trilinear interpolation. This gap can be modified with the\
            -uniform option but by default it does not extend beyond the FOV unless\
            -crop_unbound is used.
        crop_unbound: Allow padding beyond the original FOV when cropping.
        axis: pad or crop the input image along the provided axis (defined by\
            index). The specifier argument defines the number of voxels added or\
            removed on the lower or upper end of the axis (-axis index\
            delta_lower,delta_upper) or acts as a voxel selection range (-axis\
            index start:stop). In both modes, values are relative to the input\
            image (overriding all other extent-specifying options). Negative delta\
            specifier values trigger the inverse operation (pad instead of crop and\
            vice versa) and negative range specifier trigger padding. Note that the\
            deprecated commands 'mrcrop' and 'mrpad' used range-based and\
            delta-based -axis indices, respectively.
        all_axes: Crop or pad all, not just spatial axes.
        fill: Use number as the out of bounds value. nan, inf and -inf are\
            valid arguments. (Default: 0.0).
        strides: specify the strides of the output data in memory; either as a\
            comma-separated list of (signed) integers, or as a template image from\
            which the strides shall be extracted and used. The actual strides\
            produced will depend on whether the output image format can support it.
        datatype: specify output image data type. Valid choices are: float32,\
            float32le, float32be, float64, float64le, float64be, int64, uint64,\
            int64le, uint64le, int64be, uint64be, int32, uint32, int32le, uint32le,\
            int32be, uint32be, int16, uint16, int16le, uint16le, int16be, uint16be,\
            cfloat32, cfloat32le, cfloat32be, cfloat64, cfloat64le, cfloat64be,\
            int8, uint8, bit.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrgridOutputs`).
    """
    params = mrgrid_params(
        template=template,
        size=size,
        voxel=voxel,
        scale=scale,
        interp=interp,
        oversample=oversample,
        as_=as_,
        uniform=uniform,
        mask=mask,
        crop_unbound=crop_unbound,
        axis=axis,
        all_axes=all_axes,
        fill=fill,
        strides=strides,
        datatype=datatype,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
        operation=operation,
        output=output,
    )
    return mrgrid_execute(params, runner)


__all__ = [
    "MRGRID_METADATA",
    "MrgridAxisParamsDict",
    "MrgridAxisParamsDictTagged",
    "MrgridConfigParamsDict",
    "MrgridConfigParamsDictTagged",
    "MrgridOutputs",
    "MrgridParamsDict",
    "MrgridParamsDictTagged",
    "MrgridVariousFileParamsDict",
    "MrgridVariousFileParamsDictTagged",
    "MrgridVariousStringParamsDict",
    "MrgridVariousStringParamsDictTagged",
    "mrgrid",
    "mrgrid_axis",
    "mrgrid_config",
    "mrgrid_execute",
    "mrgrid_params",
    "mrgrid_various_file",
    "mrgrid_various_string",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

MRMETRIC_METADATA = Metadata(
    id="900f502c877db64aba337cc23f205ccc871706a3.boutiques",
    name="mrmetric",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_MrmetricConfigParamsDictNoTag = typing.TypedDict('_MrmetricConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
MrmetricConfigParamsDictTagged = typing.TypedDict('MrmetricConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
MrmetricConfigParamsDict = _MrmetricConfigParamsDictNoTag | MrmetricConfigParamsDictTagged


_MrmetricParamsDictNoTag = typing.TypedDict('_MrmetricParamsDictNoTag', {
    "space": typing.NotRequired[str | None],
    "interp": typing.NotRequired[str | None],
    "metric": typing.NotRequired[str | None],
    "mask1": typing.NotRequired[InputPathType | None],
    "mask2": typing.NotRequired[InputPathType | None],
    "nonormalisation": bool,
    "overlap": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrmetricConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "image1": InputPathType,
    "image2": InputPathType,
})
MrmetricParamsDictTagged = typing.TypedDict('MrmetricParamsDictTagged', {
    "@type": typing.Literal["mrtrix/mrmetric"],
    "space": typing.NotRequired[str | None],
    "interp": typing.NotRequired[str | None],
    "metric": typing.NotRequired[str | None],
    "mask1": typing.NotRequired[InputPathType | None],
    "mask2": typing.NotRequired[InputPathType | None],
    "nonormalisation": bool,
    "overlap": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrmetricConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "image1": InputPathType,
    "image2": InputPathType,
})
MrmetricParamsDict = _MrmetricParamsDictNoTag | MrmetricParamsDictTagged


def mrmetric_config(
    key: str,
    value: str,
) -> MrmetricConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def mrmetric_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrmetricConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def mrmetric_config_cargs(
    params: MrmetricConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class MrmetricOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrmetricParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def mrmetric_params(
    image1: InputPathType,
    image2: InputPathType,
    space: str | None = None,
    interp: str | None = None,
    metric: str | None = None,
    mask1: InputPathType | None = None,
    mask2: InputPathType | None = None,
    nonormalisation: bool = False,
    overlap: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrmetricConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> MrmetricParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image1: the first input image.
        image2: the second input image.
        space: voxel (default): per voxel image1: scanner space of image 1\
            image2: scanner space of image 2 average: scanner space of the average\
            affine transformation of image 1 and 2.
        interp: set the interpolation method to use when reslicing (choices:\
            nearest, linear, cubic, sinc. Default: linear).
        metric: define the dissimilarity metric used to calculate the cost.\
            Choices: diff (squared differences), cc (non-normalised negative cross\
            correlation aka negative cross covariance). Default: diff). cc is only\
            implemented for -space average and -interp linear and cubic.
        mask1: mask for image 1.
        mask2: mask for image 2.
        nonormalisation: do not normalise the dissimilarity metric to the\
            number of voxels.
        overlap: output number of voxels that were used.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/mrmetric",
        "nonormalisation": nonormalisation,
        "overlap": overlap,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "image1": image1,
        "image2": image2,
    }
    if space is not None:
        params["space"] = space
    if interp is not None:
        params["interp"] = interp
    if metric is not None:
        params["metric"] = metric
    if mask1 is not None:
        params["mask1"] = mask1
    if mask2 is not None:
        params["mask2"] = mask2
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def mrmetric_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrmetricParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("space", None) is not None:
        if not isinstance(params["space"], str):
            raise StyxValidationError(f'`space` has the wrong type: Received `{type(params.get("space", None))}` expected `str | None`')
    if params.get("interp", None) is not None:
        if not isinstance(params["interp"], str):
            raise StyxValidationError(f'`interp` has the wrong type: Received `{type(params.get("interp", None))}` expected `str | None`')
    if params.get("metric", None) is not None:
        if not isinstance(params["metric"], str):
            raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `str | None`')
    if params.get("mask1", None) is not None:
        if not isinstance(params["mask1"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask1` has the wrong type: Received `{type(params.get("mask1", None))}` expected `InputPathType | None`')
    if params.get("mask2", None) is not None:
        if not isinstance(params["mask2"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask2` has the wrong type: Received `{type(params.get("mask2", None))}` expected `InputPathType | None`')
    if params.get("nonormalisation", False) is None:
        raise StyxValidationError("`nonormalisation` must not be None")
    if not isinstance(params["nonormalisation"], bool):
        raise StyxValidationError(f'`nonormalisation` has the wrong type: Received `{type(params.get("nonormalisation", False))}` expected `bool`')
    if params.get("overlap", False) is None:
        raise StyxValidationError("`overlap` must not be None")
    if not isinstance(params["overlap"], bool):
        raise StyxValidationError(f'`overlap` has the wrong type: Received `{type(params.get("overlap", False))}` expected `bool`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[MrmetricConfigParamsDict] | None`')
        for e in params["config"]:
            mrmetric_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("image1", None) is None:
        raise StyxValidationError("`image1` must not be None")
    if not isinstance(params["image1"], (pathlib.Path, str)):
        raise StyxValidationError(f'`image1` has the wrong type: Received `{type(params.get("image1", None))}` expected `InputPathType`')
    if params.get("image2", None) is None:
        raise StyxValidationError("`image2` must not be None")
    if not isinstance(params["image2"], (pathlib.Path, str)):
        raise StyxValidationError(f'`image2` has the wrong type: Received `{type(params.get("image2", None))}` expected `InputPathType`')


def mrmetric_cargs(
    params: MrmetricParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("mrmetric")
    if params.get("space", None) is not None:
        cargs.extend([
            "-space",
            params.get("space", None)
        ])
    if params.get("interp", None) is not None:
        cargs.extend([
            "-interp",
            params.get("interp", None)
        ])
    if params.get("metric", None) is not None:
        cargs.extend([
            "-metric",
            params.get("metric", None)
        ])
    if params.get("mask1", None) is not None:
        cargs.extend([
            "-mask1",
            execution.input_file(params.get("mask1", None))
        ])
    if params.get("mask2", None) is not None:
        cargs.extend([
            "-mask2",
            execution.input_file(params.get("mask2", None))
        ])
    if params.get("nonormalisation", False):
        cargs.append("-nonormalisation")
    if params.get("overlap", False):
        cargs.append("-overlap")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [mrmetric_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("image1", None)))
    cargs.append(execution.input_file(params.get("image2", None)))
    return cargs


def mrmetric_outputs(
    params: MrmetricParamsDict,
    execution: Execution,
) -> MrmetricOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MrmetricOutputs(
        root=execution.output_file("."),
    )
    return ret


def mrmetric_execute(
    params: MrmetricParamsDict,
    runner: Runner | None = None,
) -> MrmetricOutputs:
    """
    mrmetric
    
    Computes a dissimilarity metric between two images.
    
    Currently only the mean squared difference is fully implemented.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrmetricOutputs`).
    """
    mrmetric_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRMETRIC_METADATA)
    params = execution.params(params)
    cargs = mrmetric_cargs(params, execution)
    ret = mrmetric_outputs(params, execution)
    execution.run(cargs)
    return ret


def mrmetric(
    image1: InputPathType,
    image2: InputPathType,
    space: str | None = None,
    interp: str | None = None,
    metric: str | None = None,
    mask1: InputPathType | None = None,
    mask2: InputPathType | None = None,
    nonormalisation: bool = False,
    overlap: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrmetricConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrmetricOutputs:
    """
    mrmetric
    
    Computes a dissimilarity metric between two images.
    
    Currently only the mean squared difference is fully implemented.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        image1: the first input image.
        image2: the second input image.
        space: voxel (default): per voxel image1: scanner space of image 1\
            image2: scanner space of image 2 average: scanner space of the average\
            affine transformation of image 1 and 2.
        interp: set the interpolation method to use when reslicing (choices:\
            nearest, linear, cubic, sinc. Default: linear).
        metric: define the dissimilarity metric used to calculate the cost.\
            Choices: diff (squared differences), cc (non-normalised negative cross\
            correlation aka negative cross covariance). Default: diff). cc is only\
            implemented for -space average and -interp linear and cubic.
        mask1: mask for image 1.
        mask2: mask for image 2.
        nonormalisation: do not normalise the dissimilarity metric to the\
            number of voxels.
        overlap: output number of voxels that were used.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrmetricOutputs`).
    """
    params = mrmetric_params(
        space=space,
        interp=interp,
        metric=metric,
        mask1=mask1,
        mask2=mask2,
        nonormalisation=nonormalisation,
        overlap=overlap,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        image1=image1,
        image2=image2,
    )
    return mrmetric_execute(params, runner)


__all__ = [
    "MRMETRIC_METADATA",
    "MrmetricConfigParamsDict",
    "MrmetricConfigParamsDictTagged",
    "MrmetricOutputs",
    "MrmetricParamsDict",
    "MrmetricParamsDictTagged",
    "mrmetric",
    "mrmetric_config",
    "mrmetric_execute",
    "mrmetric_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

AFDCONNECTIVITY_METADATA = Metadata(
    id="a6fb5580a7faf6e50bbcf57a2c6385623c779645.boutiques",
    name="afdconnectivity",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_AfdconnectivityConfigParamsDictNoTag = typing.TypedDict('_AfdconnectivityConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
AfdconnectivityConfigParamsDictTagged = typing.TypedDict('AfdconnectivityConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
AfdconnectivityConfigParamsDict = _AfdconnectivityConfigParamsDictNoTag | AfdconnectivityConfigParamsDictTagged


_AfdconnectivityParamsDictNoTag = typing.TypedDict('_AfdconnectivityParamsDictNoTag', {
    "wbft": typing.NotRequired[InputPathType | None],
    "afd_map": typing.NotRequired[str | None],
    "all_fixels": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[AfdconnectivityConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "image": InputPathType,
    "tracks": InputPathType,
})
AfdconnectivityParamsDictTagged = typing.TypedDict('AfdconnectivityParamsDictTagged', {
    "@type": typing.Literal["mrtrix/afdconnectivity"],
    "wbft": typing.NotRequired[InputPathType | None],
    "afd_map": typing.NotRequired[str | None],
    "all_fixels": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[AfdconnectivityConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "image": InputPathType,
    "tracks": InputPathType,
})
AfdconnectivityParamsDict = _AfdconnectivityParamsDictNoTag | AfdconnectivityParamsDictTagged


def afdconnectivity_config(
    key: str,
    value: str,
) -> AfdconnectivityConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def afdconnectivity_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AfdconnectivityConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def afdconnectivity_config_cargs(
    params: AfdconnectivityConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class AfdconnectivityOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AfdconnectivityParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    afd_map: OutputPathType | None
    """output a 3D image containing the AFD estimated for each voxel. """


def afdconnectivity_params(
    image: InputPathType,
    tracks: InputPathType,
    wbft: InputPathType | None = None,
    afd_map: str | None = None,
    all_fixels: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[AfdconnectivityConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> AfdconnectivityParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image: the input FOD image.
        tracks: the input track file defining the bundle of interest.
        wbft: provide a whole-brain fibre-tracking data set (of which the input\
            track file should be a subset), to improve the estimate of fibre bundle\
            volume in the presence of partial volume.
        afd_map: output a 3D image containing the AFD estimated for each voxel.
        all_fixels: if whole-brain fibre-tracking is NOT provided, then if\
            multiple fixels within a voxel are traversed by the pathway of\
            interest, by default the fixel with the greatest streamlines density is\
            selected to contribute to the AFD in that voxel. If this option is\
            provided, then ALL fixels with non-zero streamlines density will\
            contribute to the result, even if multiple fixels per voxel are\
            selected.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/afdconnectivity",
        "all_fixels": all_fixels,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "image": image,
        "tracks": tracks,
    }
    if wbft is not None:
        params["wbft"] = wbft
    if afd_map is not None:
        params["afd_map"] = afd_map
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def afdconnectivity_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AfdconnectivityParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("wbft", None) is not None:
        if not isinstance(params["wbft"], (pathlib.Path, str)):
            raise StyxValidationError(f'`wbft` has the wrong type: Received `{type(params.get("wbft", None))}` expected `InputPathType | None`')
    if params.get("afd_map", None) is not None:
        if not isinstance(params["afd_map"], str):
            raise StyxValidationError(f'`afd_map` has the wrong type: Received `{type(params.get("afd_map", None))}` expected `str | None`')
    if params.get("all_fixels", False) is None:
        raise StyxValidationError("`all_fixels` must not be None")
    if not isinstance(params["all_fixels"], bool):
        raise StyxValidationError(f'`all_fixels` has the wrong type: Received `{type(params.get("all_fixels", False))}` expected `bool`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[AfdconnectivityConfigParamsDict] | None`')
        for e in params["config"]:
            afdconnectivity_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("image", None) is None:
        raise StyxValidationError("`image` must not be None")
    if not isinstance(params["image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`image` has the wrong type: Received `{type(params.get("image", None))}` expected `InputPathType`')
    if params.get("tracks", None) is None:
        raise StyxValidationError("`tracks` must not be None")
    if not isinstance(params["tracks"], (pathlib.Path, str)):
        raise StyxValidationError(f'`tracks` has the wrong type: Received `{type(params.get("tracks", None))}` expected `InputPathType`')


def afdconnectivity_cargs(
    params: AfdconnectivityParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("afdconnectivity")
    if params.get("wbft", None) is not None:
        cargs.extend([
            "-wbft",
            execution.input_file(params.get("wbft", None))
        ])
    if params.get("afd_map", None) is not None:
        cargs.extend([
            "-afd_map",
            params.get("afd_map", None)
        ])
    if params.get("all_fixels", False):
        cargs.append("-all_fixels")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [afdconnectivity_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("image", None)))
    cargs.append(execution.input_file(params.get("tracks", None)))
    return cargs


def afdconnectivity_outputs(
    params: AfdconnectivityParamsDict,
    execution: Execution,
) -> AfdconnectivityOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AfdconnectivityOutputs(
        root=execution.output_file("."),
        afd_map=execution.output_file(params.get("afd_map", None)) if (params.get("afd_map") is not None) else None,
    )
    return ret


def afdconnectivity_execute(
    params: AfdconnectivityParamsDict,
    runner: Runner | None = None,
) -> AfdconnectivityOutputs:
    """
    afdconnectivity
    
    Obtain an estimate of fibre connectivity between two regions using AFD and
    streamlines tractography.
    
    This estimate is obtained by determining a fibre volume (AFD) occupied by
    the pathway of interest, and dividing by the streamline length.
    
    If only the streamlines belonging to the pathway of interest are provided,
    then ALL of the fibre volume within each fixel selected will contribute to
    the result. If the -wbft option is used to provide whole-brain
    fibre-tracking (of which the pathway of interest should contain a subset),
    only the fraction of the fibre volume in each fixel estimated to belong to
    the pathway of interest will contribute to the result.
    
    Use -quiet to suppress progress messages and output fibre connectivity value
    only.
    
    For valid comparisons of AFD connectivity across scans, images MUST be
    intensity normalised and bias field corrected, and a common response
    function for all subjects must be used.
    
    Note that the sum of the AFD is normalised by streamline length to account
    for subject differences in fibre bundle length. This normalisation results
    in a measure that is more related to the cross-sectional volume of the tract
    (and therefore 'connectivity'). Note that SIFT-ed tract count is a superior
    measure because it is unaffected by tangential yet unrelated fibres.
    However, AFD connectivity may be used as a substitute when Anatomically
    Constrained Tractography is not possible due to uncorrectable EPI
    distortions, and SIFT may therefore not be as effective.
    
    Longer discussion regarding this command can additionally be found at:
    https://mrtrix.readthedocs.io/en/3.0.4/concepts/afd_connectivity.html (as
    well as in the relevant reference).
    
    References:
    
    Smith, R. E.; Raffelt, D.; Tournier, J.-D.; Connelly, A. Quantitative
    Streamlines Tractography: Methods and Inter-Subject Normalisation. Open
    Science Framework, https://doi.org/10.31219/osf.io/c67kn.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AfdconnectivityOutputs`).
    """
    afdconnectivity_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(AFDCONNECTIVITY_METADATA)
    params = execution.params(params)
    cargs = afdconnectivity_cargs(params, execution)
    ret = afdconnectivity_outputs(params, execution)
    execution.run(cargs)
    return ret


def afdconnectivity(
    image: InputPathType,
    tracks: InputPathType,
    wbft: InputPathType | None = None,
    afd_map: str | None = None,
    all_fixels: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[AfdconnectivityConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> AfdconnectivityOutputs:
    """
    afdconnectivity
    
    Obtain an estimate of fibre connectivity between two regions using AFD and
    streamlines tractography.
    
    This estimate is obtained by determining a fibre volume (AFD) occupied by
    the pathway of interest, and dividing by the streamline length.
    
    If only the streamlines belonging to the pathway of interest are provided,
    then ALL of the fibre volume within each fixel selected will contribute to
    the result. If the -wbft option is used to provide whole-brain
    fibre-tracking (of which the pathway of interest should contain a subset),
    only the fraction of the fibre volume in each fixel estimated to belong to
    the pathway of interest will contribute to the result.
    
    Use -quiet to suppress progress messages and output fibre connectivity value
    only.
    
    For valid comparisons of AFD connectivity across scans, images MUST be
    intensity normalised and bias field corrected, and a common response
    function for all subjects must be used.
    
    Note that the sum of the AFD is normalised by streamline length to account
    for subject differences in fibre bundle length. This normalisation results
    in a measure that is more related to the cross-sectional volume of the tract
    (and therefore 'connectivity'). Note that SIFT-ed tract count is a superior
    measure because it is unaffected by tangential yet unrelated fibres.
    However, AFD connectivity may be used as a substitute when Anatomically
    Constrained Tractography is not possible due to uncorrectable EPI
    distortions, and SIFT may therefore not be as effective.
    
    Longer discussion regarding this command can additionally be found at:
    https://mrtrix.readthedocs.io/en/3.0.4/concepts/afd_connectivity.html (as
    well as in the relevant reference).
    
    References:
    
    Smith, R. E.; Raffelt, D.; Tournier, J.-D.; Connelly, A. Quantitative
    Streamlines Tractography: Methods and Inter-Subject Normalisation. Open
    Science Framework, https://doi.org/10.31219/osf.io/c67kn.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        image: the input FOD image.
        tracks: the input track file defining the bundle of interest.
        wbft: provide a whole-brain fibre-tracking data set (of which the input\
            track file should be a subset), to improve the estimate of fibre bundle\
            volume in the presence of partial volume.
        afd_map: output a 3D image containing the AFD estimated for each voxel.
        all_fixels: if whole-brain fibre-tracking is NOT provided, then if\
            multiple fixels within a voxel are traversed by the pathway of\
            interest, by default the fixel with the greatest streamlines density is\
            selected to contribute to the AFD in that voxel. If this option is\
            provided, then ALL fixels with non-zero streamlines density will\
            contribute to the result, even if multiple fixels per voxel are\
            selected.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AfdconnectivityOutputs`).
    """
    params = afdconnectivity_params(
        wbft=wbft,
        afd_map=afd_map,
        all_fixels=all_fixels,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        image=image,
        tracks=tracks,
    )
    return afdconnectivity_execute(params, runner)


__all__ = [
    "AFDCONNECTIVITY_METADATA",
    "AfdconnectivityConfigParamsDict",
    "AfdconnectivityConfigParamsDictTagged",
    "AfdconnectivityOutputs",
    "AfdconnectivityParamsDict",
    "AfdconnectivityParamsDictTagged",
    "afdconnectivity",
    "afdconnectivity_config",
    "afdconnectivity_execute",
    "afdconnectivity_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

MRINFO_METADATA = Metadata(
    id="8282cf2096dd2c6c171488cc71738cd947e2697c.boutiques",
    name="mrinfo",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_MrinfoPropertyParamsDictNoTag = typing.TypedDict('_MrinfoPropertyParamsDictNoTag', {
    "key": str,
})
MrinfoPropertyParamsDictTagged = typing.TypedDict('MrinfoPropertyParamsDictTagged', {
    "@type": typing.Literal["property"],
    "key": str,
})
MrinfoPropertyParamsDict = _MrinfoPropertyParamsDictNoTag | MrinfoPropertyParamsDictTagged


_MrinfoFslgradParamsDictNoTag = typing.TypedDict('_MrinfoFslgradParamsDictNoTag', {
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
MrinfoFslgradParamsDictTagged = typing.TypedDict('MrinfoFslgradParamsDictTagged', {
    "@type": typing.Literal["fslgrad"],
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
MrinfoFslgradParamsDict = _MrinfoFslgradParamsDictNoTag | MrinfoFslgradParamsDictTagged


_MrinfoExportGradFslParamsDictNoTag = typing.TypedDict('_MrinfoExportGradFslParamsDictNoTag', {
    "bvecs_path": str,
    "bvals_path": str,
})
MrinfoExportGradFslParamsDictTagged = typing.TypedDict('MrinfoExportGradFslParamsDictTagged', {
    "@type": typing.Literal["export_grad_fsl"],
    "bvecs_path": str,
    "bvals_path": str,
})
MrinfoExportGradFslParamsDict = _MrinfoExportGradFslParamsDictNoTag | MrinfoExportGradFslParamsDictTagged


_MrinfoExportPeEddyParamsDictNoTag = typing.TypedDict('_MrinfoExportPeEddyParamsDictNoTag', {
    "config": str,
    "indices": str,
})
MrinfoExportPeEddyParamsDictTagged = typing.TypedDict('MrinfoExportPeEddyParamsDictTagged', {
    "@type": typing.Literal["export_pe_eddy"],
    "config": str,
    "indices": str,
})
MrinfoExportPeEddyParamsDict = _MrinfoExportPeEddyParamsDictNoTag | MrinfoExportPeEddyParamsDictTagged


_MrinfoConfigParamsDictNoTag = typing.TypedDict('_MrinfoConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
MrinfoConfigParamsDictTagged = typing.TypedDict('MrinfoConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
MrinfoConfigParamsDict = _MrinfoConfigParamsDictNoTag | MrinfoConfigParamsDictTagged


_MrinfoParamsDictNoTag = typing.TypedDict('_MrinfoParamsDictNoTag', {
    "all": bool,
    "name": bool,
    "format": bool,
    "ndim": bool,
    "size": bool,
    "spacing": bool,
    "datatype": bool,
    "strides": bool,
    "offset": bool,
    "multiplier": bool,
    "transform": bool,
    "property": typing.NotRequired[list[MrinfoPropertyParamsDict] | None],
    "json_keyval": typing.NotRequired[str | None],
    "json_all": typing.NotRequired[str | None],
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[MrinfoFslgradParamsDict | None],
    "bvalue_scaling": typing.NotRequired[str | None],
    "export_grad_mrtrix": typing.NotRequired[str | None],
    "export_grad_fsl": typing.NotRequired[MrinfoExportGradFslParamsDict | None],
    "dwgrad": bool,
    "shell_bvalues": bool,
    "shell_sizes": bool,
    "shell_indices": bool,
    "export_pe_table": typing.NotRequired[str | None],
    "export_pe_eddy": typing.NotRequired[MrinfoExportPeEddyParamsDict | None],
    "petable": bool,
    "nodelete": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrinfoConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "image": list[InputPathType],
})
MrinfoParamsDictTagged = typing.TypedDict('MrinfoParamsDictTagged', {
    "@type": typing.Literal["mrtrix/mrinfo"],
    "all": bool,
    "name": bool,
    "format": bool,
    "ndim": bool,
    "size": bool,
    "spacing": bool,
    "datatype": bool,
    "strides": bool,
    "offset": bool,
    "multiplier": bool,
    "transform": bool,
    "property": typing.NotRequired[list[MrinfoPropertyParamsDict] | None],
    "json_keyval": typing.NotRequired[str | None],
    "json_all": typing.NotRequired[str | None],
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[MrinfoFslgradParamsDict | None],
    "bvalue_scaling": typing.NotRequired[str | None],
    "export_grad_mrtrix": typing.NotRequired[str | None],
    "export_grad_fsl": typing.NotRequired[MrinfoExportGradFslParamsDict | None],
    "dwgrad": bool,
    "shell_bvalues": bool,
    "shell_sizes": bool,
    "shell_indices": bool,
    "export_pe_table": typing.NotRequired[str | None],
    "export_pe_eddy": typing.NotRequired[MrinfoExportPeEddyParamsDict | None],
    "petable": bool,
    "nodelete": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrinfoConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "image": list[InputPathType],
})
MrinfoParamsDict = _MrinfoParamsDictNoTag | MrinfoParamsDictTagged


def mrinfo_property(
    key: str,
) -> MrinfoPropertyParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: any text properties embedded in the image header under the\
            specified key (use 'all' to list all keys found).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "property",
        "key": key,
    }
    return params


def mrinfo_property_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrinfoPropertyParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')


def mrinfo_property_cargs(
    params: MrinfoPropertyParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-property")
    cargs.append(params.get("key", None))
    return cargs


def mrinfo_fslgrad(
    bvecs: InputPathType,
    bvals: InputPathType,
) -> MrinfoFslgradParamsDictTagged:
    """
    Build parameters.
    
    Args:
        bvecs: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        bvals: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "fslgrad",
        "bvecs": bvecs,
        "bvals": bvals,
    }
    return params


def mrinfo_fslgrad_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrinfoFslgradParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("bvecs", None) is None:
        raise StyxValidationError("`bvecs` must not be None")
    if not isinstance(params["bvecs"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvecs` has the wrong type: Received `{type(params.get("bvecs", None))}` expected `InputPathType`')
    if params.get("bvals", None) is None:
        raise StyxValidationError("`bvals` must not be None")
    if not isinstance(params["bvals"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvals` has the wrong type: Received `{type(params.get("bvals", None))}` expected `InputPathType`')


def mrinfo_fslgrad_cargs(
    params: MrinfoFslgradParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-fslgrad")
    cargs.append(execution.input_file(params.get("bvecs", None)))
    cargs.append(execution.input_file(params.get("bvals", None)))
    return cargs


class MrinfoExportGradFslOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrinfoExportGradFslParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    bvecs_path: OutputPathType
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""
    bvals_path: OutputPathType
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""


def mrinfo_export_grad_fsl(
    bvecs_path: str,
    bvals_path: str,
) -> MrinfoExportGradFslParamsDictTagged:
    """
    Build parameters.
    
    Args:
        bvecs_path: export the diffusion-weighted gradient table to files in\
            FSL (bvecs / bvals) format.
        bvals_path: export the diffusion-weighted gradient table to files in\
            FSL (bvecs / bvals) format.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "export_grad_fsl",
        "bvecs_path": bvecs_path,
        "bvals_path": bvals_path,
    }
    return params


def mrinfo_export_grad_fsl_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrinfoExportGradFslParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("bvecs_path", None) is None:
        raise StyxValidationError("`bvecs_path` must not be None")
    if not isinstance(params["bvecs_path"], str):
        raise StyxValidationError(f'`bvecs_path` has the wrong type: Received `{type(params.get("bvecs_path", None))}` expected `str`')
    if params.get("bvals_path", None) is None:
        raise StyxValidationError("`bvals_path` must not be None")
    if not isinstance(params["bvals_path"], str):
        raise StyxValidationError(f'`bvals_path` has the wrong type: Received `{type(params.get("bvals_path", None))}` expected `str`')


def mrinfo_export_grad_fsl_cargs(
    params: MrinfoExportGradFslParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-export_grad_fsl")
    cargs.append(params.get("bvecs_path", None))
    cargs.append(params.get("bvals_path", None))
    return cargs


def mrinfo_export_grad_fsl_outputs(
    params: MrinfoExportGradFslParamsDict,
    execution: Execution,
) -> MrinfoExportGradFslOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MrinfoExportGradFslOutputs(
        root=execution.output_file("."),
        bvecs_path=execution.output_file(params.get("bvecs_path", None)),
        bvals_path=execution.output_file(params.get("bvals_path", None)),
    )
    return ret


class MrinfoExportPeEddyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrinfoExportPeEddyParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    config: OutputPathType
    """export phase-encoding information to an EDDY-style config / index file
    pair"""
    indices: OutputPathType
    """export phase-encoding information to an EDDY-style config / index file
    pair"""


def mrinfo_export_pe_eddy(
    config: str,
    indices: str,
) -> MrinfoExportPeEddyParamsDictTagged:
    """
    Build parameters.
    
    Args:
        config: export phase-encoding information to an EDDY-style config /\
            index file pair.
        indices: export phase-encoding information to an EDDY-style config /\
            index file pair.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "export_pe_eddy",
        "config": config,
        "indices": indices,
    }
    return params


def mrinfo_export_pe_eddy_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrinfoExportPeEddyParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("config", None) is None:
        raise StyxValidationError("`config` must not be None")
    if not isinstance(params["config"], str):
        raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `str`')
    if params.get("indices", None) is None:
        raise StyxValidationError("`indices` must not be None")
    if not isinstance(params["indices"], str):
        raise StyxValidationError(f'`indices` has the wrong type: Received `{type(params.get("indices", None))}` expected `str`')


def mrinfo_export_pe_eddy_cargs(
    params: MrinfoExportPeEddyParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-export_pe_eddy")
    cargs.append(params.get("config", None))
    cargs.append(params.get("indices", None))
    return cargs


def mrinfo_export_pe_eddy_outputs(
    params: MrinfoExportPeEddyParamsDict,
    execution: Execution,
) -> MrinfoExportPeEddyOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MrinfoExportPeEddyOutputs(
        root=execution.output_file("."),
        config=execution.output_file(params.get("config", None)),
        indices=execution.output_file(params.get("indices", None)),
    )
    return ret


def mrinfo_config(
    key: str,
    value: str,
) -> MrinfoConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def mrinfo_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrinfoConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def mrinfo_config_cargs(
    params: MrinfoConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class MrinfoOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrinfoParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    json_keyval: OutputPathType | None
    """export header key/value entries to a JSON file """
    json_all: OutputPathType | None
    """export all header contents to a JSON file """
    export_grad_mrtrix: OutputPathType | None
    """export the diffusion-weighted gradient table to file in MRtrix format """
    export_pe_table: OutputPathType | None
    """export phase-encoding table to file """
    export_grad_fsl: MrinfoExportGradFslOutputs | None
    """Outputs from `mrinfo_export_grad_fsl_outputs`."""
    export_pe_eddy: MrinfoExportPeEddyOutputs | None
    """Outputs from `mrinfo_export_pe_eddy_outputs`."""


def mrinfo_params(
    image: list[InputPathType],
    all_: bool = False,
    name: bool = False,
    format_: bool = False,
    ndim: bool = False,
    size: bool = False,
    spacing: bool = False,
    datatype: bool = False,
    strides: bool = False,
    offset: bool = False,
    multiplier: bool = False,
    transform: bool = False,
    property_: list[MrinfoPropertyParamsDict] | None = None,
    json_keyval: str | None = None,
    json_all: str | None = None,
    grad: InputPathType | None = None,
    fslgrad: MrinfoFslgradParamsDict | None = None,
    bvalue_scaling: str | None = None,
    export_grad_mrtrix: str | None = None,
    export_grad_fsl: MrinfoExportGradFslParamsDict | None = None,
    dwgrad: bool = False,
    shell_bvalues: bool = False,
    shell_sizes: bool = False,
    shell_indices: bool = False,
    export_pe_table: str | None = None,
    export_pe_eddy: MrinfoExportPeEddyParamsDict | None = None,
    petable: bool = False,
    nodelete: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrinfoConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> MrinfoParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image: the input image(s).
        all_: print all properties, rather than the first and last 2 of each.
        name: print the file system path of the image.
        format_: image file format.
        ndim: number of image dimensions.
        size: image size along each axis.
        spacing: voxel spacing along each image dimension.
        datatype: data type used for image data storage.
        strides: data strides i.e. order and direction of axes data layout.
        offset: image intensity offset.
        multiplier: image intensity multiplier.
        transform: the transformation from image coordinates [mm] to scanner /\
            real world coordinates [mm].
        property_: any text properties embedded in the image header under the\
            specified key (use 'all' to list all keys found).
        json_keyval: export header key/value entries to a JSON file.
        json_all: export all header contents to a JSON file.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        bvalue_scaling: enable or disable scaling of diffusion b-values by the\
            square of the corresponding DW gradient norm (see Desciption). Valid\
            choices are yes/no, true/false, 0/1 (default: automatic).
        export_grad_mrtrix: export the diffusion-weighted gradient table to\
            file in MRtrix format.
        export_grad_fsl: export the diffusion-weighted gradient table to files\
            in FSL (bvecs / bvals) format.
        dwgrad: the diffusion-weighting gradient table, as interpreted by\
            MRtrix3.
        shell_bvalues: list the average b-value of each shell.
        shell_sizes: list the number of volumes in each shell.
        shell_indices: list the image volumes attributed to each b-value shell.
        export_pe_table: export phase-encoding table to file.
        export_pe_eddy: export phase-encoding information to an EDDY-style\
            config / index file pair.
        petable: print the phase encoding table.
        nodelete: don't delete temporary images or images passed to mrinfo via\
            Unix pipes.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/mrinfo",
        "all": all_,
        "name": name,
        "format": format_,
        "ndim": ndim,
        "size": size,
        "spacing": spacing,
        "datatype": datatype,
        "strides": strides,
        "offset": offset,
        "multiplier": multiplier,
        "transform": transform,
        "dwgrad": dwgrad,
        "shell_bvalues": shell_bvalues,
        "shell_sizes": shell_sizes,
        "shell_indices": shell_indices,
        "petable": petable,
        "nodelete": nodelete,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "image": image,
    }
    if property_ is not None:
        params["property"] = property_
    if json_keyval is not None:
        params["json_keyval"] = json_keyval
    if json_all is not None:
        params["json_all"] = json_all
    if grad is not None:
        params["grad"] = grad
    if fslgrad is not None:
        params["fslgrad"] = fslgrad
    if bvalue_scaling is not None:
        params["bvalue_scaling"] = bvalue_scaling
    if export_grad_mrtrix is not None:
        params["export_grad_mrtrix"] = export_grad_mrtrix
    if export_grad_fsl is not None:
        params["export_grad_fsl"] = export_grad_fsl
    if export_pe_table is not None:
        params["export_pe_table"] = export_pe_table
    if export_pe_eddy is not None:
        params["export_pe_eddy"] = export_pe_eddy
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def mrinfo_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrinfoParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("all", False) is None:
        raise StyxValidationError("`all` must not be None")
    if not isinstance(params["all"], bool):
        raise StyxValidationError(f'`all` has the wrong type: Received `{type(params.get("all", False))}` expected `bool`')
    if params.get("name", False) is None:
        raise StyxValidationError("`name` must not be None")
    if not isinstance(params["name"], bool):
        raise StyxValidationError(f'`name` has the wrong type: Received `{type(params.get("name", False))}` expected `bool`')
    if params.get("format", False) is None:
        raise StyxValidationError("`format` must not be None")
    if not isinstance(params["format"], bool):
        raise StyxValidationError(f'`format` has the wrong type: Received `{type(params.get("format", False))}` expected `bool`')
    if params.get("ndim", False) is None:
        raise StyxValidationError("`ndim` must not be None")
    if not isinstance(params["ndim"], bool):
        raise StyxValidationError(f'`ndim` has the wrong type: Received `{type(params.get("ndim", False))}` expected `bool`')
    if params.get("size", False) is None:
        raise StyxValidationError("`size` must not be None")
    if not isinstance(params["size"], bool):
        raise StyxValidationError(f'`size` has the wrong type: Received `{type(params.get("size", False))}` expected `bool`')
    if params.get("spacing", False) is None:
        raise StyxValidationError("`spacing` must not be None")
    if not isinstance(params["spacing"], bool):
        raise StyxValidationError(f'`spacing` has the wrong type: Received `{type(params.get("spacing", False))}` expected `bool`')
    if params.get("datatype", False) is None:
        raise StyxValidationError("`datatype` must not be None")
    if not isinstance(params["datatype"], bool):
        raise StyxValidationError(f'`datatype` has the wrong type: Received `{type(params.get("datatype", False))}` expected `bool`')
    if params.get("strides", False) is None:
        raise StyxValidationError("`strides` must not be None")
    if not isinstance(params["strides"], bool):
        raise StyxValidationError(f'`strides` has the wrong type: Received `{type(params.get("strides", False))}` expected `bool`')
    if params.get("offset", False) is None:
        raise StyxValidationError("`offset` must not be None")
    if not isinstance(params["offset"], bool):
        raise StyxValidationError(f'`offset` has the wrong type: Received `{type(params.get("offset", False))}` expected `bool`')
    if params.get("multiplier", False) is None:
        raise StyxValidationError("`multiplier` must not be None")
    if not isinstance(params["multiplier"], bool):
        raise StyxValidationError(f'`multiplier` has the wrong type: Received `{type(params.get("multiplier", False))}` expected `bool`')
    if params.get("transform", False) is None:
        raise StyxValidationError("`transform` must not be None")
    if not isinstance(params["transform"], bool):
        raise StyxValidationError(f'`transform` has the wrong type: Received `{type(params.get("transform", False))}` expected `bool`')
    if params.get("property", None) is not None:
        if not isinstance(params["property"], list):
            raise StyxValidationError(f'`property` has the wrong type: Received `{type(params.get("property", None))}` expected `list[MrinfoPropertyParamsDict] | None`')
        for e in params["property"]:
            mrinfo_property_validate(e)
    if params.get("json_keyval", None) is not None:
        if not isinstance(params["json_keyval"], str):
            raise StyxValidationError(f'`json_keyval` has the wrong type: Received `{type(params.get("json_keyval", None))}` expected `str | None`')
    if params.get("json_all", None) is not None:
        if not isinstance(params["json_all"], str):
            raise StyxValidationError(f'`json_all` has the wrong type: Received `{type(params.get("json_all", None))}` expected `str | None`')
    if params.get("grad", None) is not None:
        if not isinstance(params["grad"], (pathlib.Path, str)):
            raise StyxValidationError(f'`grad` has the wrong type: Received `{type(params.get("grad", None))}` expected `InputPathType | None`')
    if params.get("fslgrad", None) is not None:
        mrinfo_fslgrad_validate(params["fslgrad"])
    if params.get("bvalue_scaling", None) is not None:
        if not isinstance(params["bvalue_scaling"], str):
            raise StyxValidationError(f'`bvalue_scaling` has the wrong type: Received `{type(params.get("bvalue_scaling", None))}` expected `str | None`')
    if params.get("export_grad_mrtrix", None) is not None:
        if not isinstance(params["export_grad_mrtrix"], str):
            raise StyxValidationError(f'`export_grad_mrtrix` has the wrong type: Received `{type(params.get("export_grad_mrtrix", None))}` expected `str | None`')
    if params.get("export_grad_fsl", None) is not None:
        mrinfo_export_grad_fsl_validate(params["export_grad_fsl"])
    if params.get("dwgrad", False) is None:
        raise StyxValidationError("`dwgrad` must not be None")
    if not isinstance(params["dwgrad"], bool):
        raise StyxValidationError(f'`dwgrad` has the wrong type: Received `{type(params.get("dwgrad", False))}` expected `bool`')
    if params.get("shell_bvalues", False) is None:
        raise StyxValidationError("`shell_bvalues` must not be None")
    if not isinstance(params["shell_bvalues"], bool):
        raise StyxValidationError(f'`shell_bvalues` has the wrong type: Received `{type(params.get("shell_bvalues", False))}` expected `bool`')
    if params.get("shell_sizes", False) is None:
        raise StyxValidationError("`shell_sizes` must not be None")
    if not isinstance(params["shell_sizes"], bool):
        raise StyxValidationError(f'`shell_sizes` has the wrong type: Received `{type(params.get("shell_sizes", False))}` expected `bool`')
    if params.get("shell_indices", False) is None:
        raise StyxValidationError("`shell_indices` must not be None")
    if not isinstance(params["shell_indices"], bool):
        raise StyxValidationError(f'`shell_indices` has the wrong type: Received `{type(params.get("shell_indices", False))}` expected `bool`')
    if params.get("export_pe_table", None) is not None:
        if not isinstance(params["export_pe_table"], str):
            raise StyxValidationError(f'`export_pe_table` has the wrong type: Received `{type(params.get("export_pe_table", None))}` expected `str | None`')
    if params.get("export_pe_eddy", None) is not None:
        mrinfo_export_pe_eddy_validate(params["export_pe_eddy"])
    if params.get("petable", False) is None:
        raise StyxValidationError("`petable` must not be None")
    if not isinstance(params["petable"], bool):
        raise StyxValidationError(f'`petable` has the wrong type: Received `{type(params.get("petable", False))}` expected `bool`')
    if params.get("nodelete", False) is None:
        raise StyxValidationError("`nodelete` must not be None")
    if not isinstance(params["nodelete"], bool):
        raise StyxValidationError(f'`nodelete` has the wrong type: Received `{type(params.get("nodelete", False))}` expected `bool`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[MrinfoConfigParamsDict] | None`')
        for e in params["config"]:
            mrinfo_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("image", None) is None:
        raise StyxValidationError("`image` must not be None")
    if not isinstance(params["image"], list):
        raise StyxValidationError(f'`image` has the wrong type: Received `{type(params.get("image", None))}` expected `list[InputPathType]`')
    for e in params["image"]:
        if not isinstance(e, (pathlib.Path, str)):
            raise StyxValidationError(f'`image` has the wrong type: Received `{type(params.get("image", None))}` expected `list[InputPathType]`')


def mrinfo_cargs(
    params: MrinfoParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("mrinfo")
    if params.get("all", False):
        cargs.append("-all")
    if params.get("name", False):
        cargs.append("-name")
    if params.get("format", False):
        cargs.append("-format")
    if params.get("ndim", False):
        cargs.append("-ndim")
    if params.get("size", False):
        cargs.append("-size")
    if params.get("spacing", False):
        cargs.append("-spacing")
    if params.get("datatype", False):
        cargs.append("-datatype")
    if params.get("strides", False):
        cargs.append("-strides")
    if params.get("offset", False):
        cargs.append("-offset")
    if params.get("multiplier", False):
        cargs.append("-multiplier")
    if params.get("transform", False):
        cargs.append("-transform")
    if params.get("property", None) is not None:
        cargs.extend([a for c in [mrinfo_property_cargs(s, execution) for s in params.get("property", None)] for a in c])
    if params.get("json_keyval", None) is not None:
        cargs.extend([
            "-json_keyval",
            params.get("json_keyval", None)
        ])
    if params.get("json_all", None) is not None:
        cargs.extend([
            "-json_all",
            params.get("json_all", None)
        ])
    if params.get("grad", None) is not None:
        cargs.extend([
            "-grad",
            execution.input_file(params.get("grad", None))
        ])
    if params.get("fslgrad", None) is not None:
        cargs.extend(mrinfo_fslgrad_cargs(params.get("fslgrad", None), execution))
    if params.get("bvalue_scaling", None) is not None:
        cargs.extend([
            "-bvalue_scaling",
            params.get("bvalue_scaling", None)
        ])
    if params.get("export_grad_mrtrix", None) is not None:
        cargs.extend([
            "-export_grad_mrtrix",
            params.get("export_grad_mrtrix", None)
        ])
    if params.get("export_grad_fsl", None) is not None:
        cargs.extend(mrinfo_export_grad_fsl_cargs(params.get("export_grad_fsl", None), execution))
    if params.get("dwgrad", False):
        cargs.append("-dwgrad")
    if params.get("shell_bvalues", False):
        cargs.append("-shell_bvalues")
    if params.get("shell_sizes", False):
        cargs.append("-shell_sizes")
    if params.get("shell_indices", False):
        cargs.append("-shell_indices")
    if params.get("export_pe_table", None) is not None:
        cargs.extend([
            "-export_pe_table",
            params.get("export_pe_table", None)
        ])
    if params.get("export_pe_eddy", None) is not None:
        cargs.extend(mrinfo_export_pe_eddy_cargs(params.get("export_pe_eddy", None), execution))
    if params.get("petable", False):
        cargs.append("-petable")
    if params.get("nodelete", False):
        cargs.append("-nodelete")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [mrinfo_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.extend([execution.input_file(f) for f in params.get("image", None)])
    return cargs


def mrinfo_outputs(
    params: MrinfoParamsDict,
    execution: Execution,
) -> MrinfoOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MrinfoOutputs(
        root=execution.output_file("."),
        json_keyval=execution.output_file(params.get("json_keyval", None)) if (params.get("json_keyval") is not None) else None,
        json_all=execution.output_file(params.get("json_all", None)) if (params.get("json_all") is not None) else None,
        export_grad_mrtrix=execution.output_file(params.get("export_grad_mrtrix", None)) if (params.get("export_grad_mrtrix") is not None) else None,
        export_pe_table=execution.output_file(params.get("export_pe_table", None)) if (params.get("export_pe_table") is not None) else None,
        export_grad_fsl=mrinfo_export_grad_fsl_outputs(params.get("export_grad_fsl"), execution) if params.get("export_grad_fsl") else None,
        export_pe_eddy=mrinfo_export_pe_eddy_outputs(params.get("export_pe_eddy"), execution) if params.get("export_pe_eddy") else None,
    )
    return ret


def mrinfo_execute(
    params: MrinfoParamsDict,
    runner: Runner | None = None,
) -> MrinfoOutputs:
    """
    mrinfo
    
    Display image header information, or extract specific information from the
    header.
    
    By default, all information contained in each image header will be printed
    to the console in a reader-friendly format.
    
    Alternatively, command-line options may be used to extract specific details
    from the header(s); these are printed to the console in a format more
    appropriate for scripting purposes or piping to file. If multiple options
    and/or images are provided, the requested header fields will be printed in
    the order in which they appear in the help page, with all requested details
    from each input image in sequence printed before the next image is
    processed.
    
    The command can also write the diffusion gradient table from a single input
    image to file; either in the MRtrix or FSL format (bvecs/bvals file pair;
    includes appropriate diffusion gradient vector reorientation)
    
    The -dwgrad, -export_* and -shell_* options provide (information about) the
    diffusion weighting gradient table after it has been processed by the
    MRtrix3 back-end (vectors normalised, b-values scaled by the square of the
    vector norm, depending on the -bvalue_scaling option). To see the raw
    gradient table information as stored in the image header, i.e. without
    MRtrix3 back-end processing, use "-property dw_scheme".
    
    The -bvalue_scaling option controls an aspect of the import of diffusion
    gradient tables. When the input diffusion-weighting direction vectors have
    norms that differ substantially from unity, the b-values will be scaled by
    the square of their corresponding vector norm (this is how multi-shell
    acquisitions are frequently achieved on scanner platforms). However in some
    rare instances, the b-values may be correct, despite the vectors not being
    of unit norm (or conversely, the b-values may need to be rescaled even
    though the vectors are close to unit norm). This option allows the user to
    control this operation and override MRrtix3's automatic detection.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrinfoOutputs`).
    """
    mrinfo_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRINFO_METADATA)
    params = execution.params(params)
    cargs = mrinfo_cargs(params, execution)
    ret = mrinfo_outputs(params, execution)
    execution.run(cargs)
    return ret


def mrinfo(
    image: list[InputPathType],
    all_: bool = False,
    name: bool = False,
    format_: bool = False,
    ndim: bool = False,
    size: bool = False,
    spacing: bool = False,
    datatype: bool = False,
    strides: bool = False,
    offset: bool = False,
    multiplier: bool = False,
    transform: bool = False,
    property_: list[MrinfoPropertyParamsDict] | None = None,
    json_keyval: str | None = None,
    json_all: str | None = None,
    grad: InputPathType | None = None,
    fslgrad: MrinfoFslgradParamsDict | None = None,
    bvalue_scaling: str | None = None,
    export_grad_mrtrix: str | None = None,
    export_grad_fsl: MrinfoExportGradFslParamsDict | None = None,
    dwgrad: bool = False,
    shell_bvalues: bool = False,
    shell_sizes: bool = False,
    shell_indices: bool = False,
    export_pe_table: str | None = None,
    export_pe_eddy: MrinfoExportPeEddyParamsDict | None = None,
    petable: bool = False,
    nodelete: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrinfoConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrinfoOutputs:
    """
    mrinfo
    
    Display image header information, or extract specific information from the
    header.
    
    By default, all information contained in each image header will be printed
    to the console in a reader-friendly format.
    
    Alternatively, command-line options may be used to extract specific details
    from the header(s); these are printed to the console in a format more
    appropriate for scripting purposes or piping to file. If multiple options
    and/or images are provided, the requested header fields will be printed in
    the order in which they appear in the help page, with all requested details
    from each input image in sequence printed before the next image is
    processed.
    
    The command can also write the diffusion gradient table from a single input
    image to file; either in the MRtrix or FSL format (bvecs/bvals file pair;
    includes appropriate diffusion gradient vector reorientation)
    
    The -dwgrad, -export_* and -shell_* options provide (information about) the
    diffusion weighting gradient table after it has been processed by the
    MRtrix3 back-end (vectors normalised, b-values scaled by the square of the
    vector norm, depending on the -bvalue_scaling option). To see the raw
    gradient table information as stored in the image header, i.e. without
    MRtrix3 back-end processing, use "-property dw_scheme".
    
    The -bvalue_scaling option controls an aspect of the import of diffusion
    gradient tables. When the input diffusion-weighting direction vectors have
    norms that differ substantially from unity, the b-values will be scaled by
    the square of their corresponding vector norm (this is how multi-shell
    acquisitions are frequently achieved on scanner platforms). However in some
    rare instances, the b-values may be correct, despite the vectors not being
    of unit norm (or conversely, the b-values may need to be rescaled even
    though the vectors are close to unit norm). This option allows the user to
    control this operation and override MRrtix3's automatic detection.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        image: the input image(s).
        all_: print all properties, rather than the first and last 2 of each.
        name: print the file system path of the image.
        format_: image file format.
        ndim: number of image dimensions.
        size: image size along each axis.
        spacing: voxel spacing along each image dimension.
        datatype: data type used for image data storage.
        strides: data strides i.e. order and direction of axes data layout.
        offset: image intensity offset.
        multiplier: image intensity multiplier.
        transform: the transformation from image coordinates [mm] to scanner /\
            real world coordinates [mm].
        property_: any text properties embedded in the image header under the\
            specified key (use 'all' to list all keys found).
        json_keyval: export header key/value entries to a JSON file.
        json_all: export all header contents to a JSON file.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        bvalue_scaling: enable or disable scaling of diffusion b-values by the\
            square of the corresponding DW gradient norm (see Desciption). Valid\
            choices are yes/no, true/false, 0/1 (default: automatic).
        export_grad_mrtrix: export the diffusion-weighted gradient table to\
            file in MRtrix format.
        export_grad_fsl: export the diffusion-weighted gradient table to files\
            in FSL (bvecs / bvals) format.
        dwgrad: the diffusion-weighting gradient table, as interpreted by\
            MRtrix3.
        shell_bvalues: list the average b-value of each shell.
        shell_sizes: list the number of volumes in each shell.
        shell_indices: list the image volumes attributed to each b-value shell.
        export_pe_table: export phase-encoding table to file.
        export_pe_eddy: export phase-encoding information to an EDDY-style\
            config / index file pair.
        petable: print the phase encoding table.
        nodelete: don't delete temporary images or images passed to mrinfo via\
            Unix pipes.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrinfoOutputs`).
    """
    params = mrinfo_params(
        all_=all_,
        name=name,
        format_=format_,
        ndim=ndim,
        size=size,
        spacing=spacing,
        datatype=datatype,
        strides=strides,
        offset=offset,
        multiplier=multiplier,
        transform=transform,
        property_=property_,
        json_keyval=json_keyval,
        json_all=json_all,
        grad=grad,
        fslgrad=fslgrad,
        bvalue_scaling=bvalue_scaling,
        export_grad_mrtrix=export_grad_mrtrix,
        export_grad_fsl=export_grad_fsl,
        dwgrad=dwgrad,
        shell_bvalues=shell_bvalues,
        shell_sizes=shell_sizes,
        shell_indices=shell_indices,
        export_pe_table=export_pe_table,
        export_pe_eddy=export_pe_eddy,
        petable=petable,
        nodelete=nodelete,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        image=image,
    )
    return mrinfo_execute(params, runner)


__all__ = [
    "MRINFO_METADATA",
    "MrinfoConfigParamsDict",
    "MrinfoConfigParamsDictTagged",
    "MrinfoExportGradFslOutputs",
    "MrinfoExportGradFslParamsDict",
    "MrinfoExportGradFslParamsDictTagged",
    "MrinfoExportPeEddyOutputs",
    "MrinfoExportPeEddyParamsDict",
    "MrinfoExportPeEddyParamsDictTagged",
    "MrinfoFslgradParamsDict",
    "MrinfoFslgradParamsDictTagged",
    "MrinfoOutputs",
    "MrinfoParamsDict",
    "MrinfoParamsDictTagged",
    "MrinfoPropertyParamsDict",
    "MrinfoPropertyParamsDictTagged",
    "mrinfo",
    "mrinfo_config",
    "mrinfo_execute",
    "mrinfo_export_grad_fsl",
    "mrinfo_export_pe_eddy",
    "mrinfo_fslgrad",
    "mrinfo_params",
    "mrinfo_property",
]
